@@ -12,9 +12,11 @@
 
 use std::marker::PhantomData;
 use std::mem;
+use std::ops::{Bound, RangeBounds};
 
 pub use willow_codegen::Program;
 
+pub use anyhow::{self, Error, Result};
 #[doc(hidden)]
 pub use field_offset::offset_of;
 #[doc(hidden)]
@@ -26,6 +28,9 @@ pub use web_sys::{
     WebGlBuffer, WebGlProgram, WebGlRenderingContext, WebGlShader, WebGlUniformLocation,
 };
 
+mod index;
+pub use index::*;
+
 mod types;
 pub use types::*;
 
@@ -43,17 +48,22 @@ pub struct Context {
 
 impl Context {
     /// Creates a context on the canvas element.
-    pub fn from_canvas(canvas: web_sys::Element) -> Option<Self> {
+    pub fn from_canvas(canvas: web_sys::Element) -> Result<Self> {
+        use anyhow::Context;
         use wasm_bindgen::JsCast;
 
-        Some(Self {
+        Ok(Self {
             native: canvas
                 .dyn_into::<web_sys::HtmlCanvasElement>()
-                .ok()?
+                .ok()
+                .context("The element is not a <canvas>")?
                 .get_context("webgl")
-                .ok()??
+                .ok()
+                .flatten()
+                .context("Could not initialize WebGL context")?
                 .dyn_into()
-                .ok()?,
+                .ok()
+                .context("WebGL context has an incorrect type")?,
         })
     }
 }
@@ -191,3 +201,20 @@ impl RenderPrimitiveType {
         }
     }
 }
+
+fn resolve_range(items: impl RangeBounds<usize>, len: usize) -> (i32, i32) {
+    let start = match items.start_bound() {
+        Bound::Included(&x) => x as i32,
+        Bound::Excluded(&x) => x as i32 - 1,
+        Bound::Unbounded => 0,
+    };
+    let end = match items.end_bound() {
+        Bound::Included(&x) => x as i32 + 1,
+        Bound::Excluded(&x) => x as i32,
+        Bound::Unbounded => len as i32,
+    };
+
+    assert!(end <= len as i32, "items range exceeds buffer size");
+
+    (start, end)
+}
